-- stylua: ignore start
import 'CoreLibs/animation.lua'
import 'CoreLibs/animator.lua'
import 'CoreLibs/easing.lua'
import 'CoreLibs/graphics.lua'
import 'CoreLibs/object.lua'
import 'CoreLibs/timer.lua'
import 'CoreLibs/ui.lua'
import 'CoreLibs/utilities/where.lua'
-- stylua: ignore end

import("../lib/tiny.lua")
import("tiny-tools.lua")
import("systems/filter-types.lua")
import("systems/collision-detection.lua")
import("systems/collision-resolution.lua")
import("systems/draw.lua")
import("systems/gravity.lua")
import("systems/velocity.lua")
import("ingredients/booster.lua")
import("ingredients/tomato.lua")
import("burger.lua")

local tiny <const> = tiny
local gfx <const> = playdate.graphics
playdate.display.setRefreshRate(50)
gfx.setBackgroundColor(gfx.kColorWhite)

burger = Burger.new()
local floorSize = { x = 10000, y = 10 }
floor = {
    position = { x = 0, y = 235 },
    size = floorSize,
    canBounce = {
        flat = { x = 0, y = 0 },
        mult = { x = 0.9, y = -0.5 },
    },
    drawAsRectangle = { size = floorSize },
    isSolid = true,
}

Camera = {
    pan = {
        x = 0,
        y = 0,
    },
}

Score = {
    points = 0,
}

-- TODO: Shops with random upgrades instead of fixed ones.

function Score:add(add)
    self.points = self.points + add
end

function Score:draw()
    gfx.drawText("Z|" .. self.points, 360, 5)
end

world = tiny.world(
    fallSystem,
    velocitySystem,
    collisionResolution,
    collisionDetection,
    drawSystem,
    drawRectanglesSystem,

    burger,
    floor
)

for i = 1, Booster.cacheSize() do
    world:addEntity(Booster.get(i * 60 + (math.random(0, 50)), math.random(50, 200)))
end

function playdate.update()
    local deltaSeconds = playdate.getElapsedTime()
    playdate.resetElapsedTime()
    gfx.clear(gfx.kColorWhite)
    playdate.drawFPS(5, 5)

    Camera.pan.x = math.min(0, -burger.position.x + 200)
    Camera.pan.y = math.max(0, -burger.position.y + 120)
    local newestX = Booster.newestBooster().position.x
    local panX = -Camera.pan.x
    local offset = 600
    if newestX + 300 < panX then
        if math.random() > 0.5 then
            world:addEntity(Tomato.get(panX + offset + (math.random(0, 50)), math.random(-500, 150)))
        else
            -- Implicitly updates cached boosters
            Booster.get(panX + offset + (math.random(0, 50)), math.random(-500, 150))
        end
    end
    floor.position.x = -Camera.pan.x - offset
    gfx.setDrawOffset(Camera.pan.x, Camera.pan.y)

    world:update(deltaSeconds)

    gfx.setDrawOffset(0, 0)
    Score:draw()

    if playdate.buttonJustPressed(playdate.kButtonA) then
        Burger.reset(burger)
        for i = 1, Booster.cacheSize() do
            Booster.get(i * 60 + (math.random(0, 50)), math.random(50, 200))
        end
    end
end
